@@ -14,7 +14,7 @@ use URI;
 use WebService::XING::Function;
 use WebService::XING::Response;
 
-our $VERSION = '0.002';
+our $VERSION = '0.020';
 
 our @CARP_NOT = qw(Mo::builder Mo::chain Mo::is Mo::required);
 @Carp::Internal{qw(Mo::builder Mo::chain Mo::is Mo::required)} = (1, 1, 1, 1);
@@ -29,6 +29,18 @@ my @FUNCTAB = (
     # User Profiles
     get_user_details =>
         [GET => '/v1/users/:id', '@fields'],
+    find_by_emails =>
+        [GET => '/v1/users/find_by_emails', '@!emails', '@user_fields'],
+
+    # Messages
+    list_conversations =>
+        [GET => '/v1/users/:user_id/conversations', 'limit', 'offset', '@user_fields', 'with_latest_messages'],
+    get_conversation =>
+        [GET => '/v1/users/:user_id/conversations/:id', '@user_fields', 'with_latest_messages'],
+    list_conversation_messages =>
+        [GET => '/v1/users/:user_id/conversations/:conversation_id/messages', 'limit', 'offset', '@user_fields'],
+    get_conversation_message =>
+        [GET => '/v1/users/:user_id/conversations/:conversation_id/messages/:id', '@user_fields'],
 
     # Status Messages
     create_status_message =>
@@ -41,15 +53,15 @@ my @FUNCTAB = (
         [PUT => '/v1/users/:user_id/profile_message', '!message', '?public=1'],
 
     # Contacts
-    get_contacts =>
+    list_contacts =>
         [GET => '/v1/users/:user_id/contacts', 'limit', 'offset', 'order_by', '@user_fields'],
-    get_shared_contacts =>
+    list_shared_contacts =>
         [GET => '/v1/users/:user_id/contacts/shared', 'limit', 'offset', 'order_by', '@user_fields'],
 
     # Contact Requests
-    get_incoming_contact_requests =>
+    list_incoming_contact_requests =>
         [GET => '/v1/users/:user_id/contact_requests', 'limit', 'offset', '@user_fields'],
-    get_sent_contact_requests =>
+    list_sent_contact_requests =>
         [GET => '/v1/users/:user_id/contact_requests/sent', 'limit', 'offset'],
     create_contact_request =>
         [POST => '/v1/users/:user_id/contact_requests', 'message'],
@@ -63,7 +75,7 @@ my @FUNCTAB = (
         [GET => '/v1/users/:user_id/network/:other_user_id/paths', '?all_paths=0', '@user_fields'],
 
     # Bookmarks
-    get_bookmarks =>
+    list_bookmarks =>
         [GET => '/v1/users/:user_id/bookmarks', 'limit', 'offset', '@user_fields'],
     create_bookmark =>
         [PUT => '/v1/users/:user_id/bookmarks/:id'],
@@ -81,13 +93,13 @@ my @FUNCTAB = (
         [POST => '/v1/activities/:id/share', 'text'],
     delete_activity =>
         [DELETE => '/v1/activities/:id'],
-    get_activity_comments =>
+    list_activity_comments =>
         [GET => '/v1/activities/:activity_id/comments', 'limit', 'offset', '@user_fields'],
     create_activity_comment =>
         [POST => '/v1/activities/:activity_id/comments', 'text'],
     delete_activity_comment =>
         [DELETE => '/v1/activities/:activity_id/comments/:id'],
-    get_activity_likes =>
+    list_activity_likes =>
         [GET => '/v1/activities/:activity_id/likes', 'limit', 'offset', '@user_fields'],
     create_activity_like =>
         [PUT => '/v1/activities/:activity_id/like'],
@@ -95,15 +107,15 @@ my @FUNCTAB = (
         [DELETE => '/v1/activities/:activity_id/like'],
 
     # Profile Visits
-    get_profile_visits =>
+    list_profile_visits =>
         [GET => '/v1/users/:user_id/visits', 'limit', 'offset', 'since', '?strip_html=0'],
     create_profile_visit =>
         [POST => '/v1/users/:user_id/visits'],
 
     # Recommendations
-    get_recommendations =>
+    list_recommendations =>
         [GET => '/v1/users/:user_id/network/recommendations', 'limit', 'offset', 'similar_user_id', '@user_fields'],
-    block_recommendation =>
+    delete_recommendation =>
         [DELETE => '/v1/users/:user_id/network/recommendations/user/:id'],
 
     # Invitations
@@ -113,7 +125,7 @@ my @FUNCTAB = (
     # Geo Locations
     update_geo_location =>
         [PUT => '/v1/users/:user_id/geo_location', '!accuracy', '!latitude', '!longitude', 'ttl'],
-    get_nearby_users  =>
+    list_nearby_users  =>
         [GET => '/v1/users/:user_id/nearby_users', 'age', 'radius', '@user_fields'],
 );
 
@@ -404,6 +416,20 @@ sub request {
     );
 }
 
+### Deprecated (renamed) methods
+
+sub get_contacts { shift->list_contacts(@_) }
+sub get_shared_contacts { shift->list_shared_contacts(@_) }
+sub get_incoming_contact_requests { shift->list_incoming_contact_requests(@_) }
+sub get_sent_contact_requests { shift->list_sent_contact_requests(@_) }
+sub get_bookmarks { shift->list_bookmarks(@_) }
+sub get_activity_comments { shift->list_activity_comments(@_) }
+sub get_activity_likes { shift->list_activity_likes(@_) }
+sub get_profile_visits { shift->list_profile_visits(@_) }
+sub get_recommendations { shift->list_recommendations(@_) }
+sub block_recommendation { shift->delete_recommendation(@_) }
+sub get_nearby_users { shift->list_nearby_users(@_) }
+
 ### Internal
 
 # ($expanded_resource, @parameters) =
@@ -423,6 +449,8 @@ sub _scour_args {
                 $self->die->(_invalid_parameter($key, ref $self, $f->name))
                     unless $p->is_list;
                 $value = join(',', @$value);
+                $self->die->(_missing_parameter($key, ref $self, $f->name))
+                    if length $value == 0 and $p->is_required;
             }
             elsif ($p->is_boolean) {
                 $value = $value && $value ne 'false' ? 'true' : 'false';
@@ -471,7 +499,7 @@ WebService::XING - Perl Interface to the XING API
 
 =head1 VERSION
 
-Version 0.002
+Version 0.020
 
 =head1 SYNOPSIS
 
@@ -854,6 +882,50 @@ session, rather than the whole L<WebService::XING> object.
 
 See L<https://dev.xing.com/docs/get/users/:id>
 
+=head2 find_user_by_email_address
+
+  $res = $xing->find_by_emails(
+    emails => \@emails, user_fields => \@user_fields
+  );
+
+See L<https://dev.xing.com/docs/get/users/find_by_emails>
+
+=head2 list_conversations
+
+  $res = $xing->list_conversations(
+    user_id => $user_id, limit => $limit, offset => $offset,
+    user_fields => \@user_fields, with_latest_messages => $number
+  );
+
+See L<https://dev.xing.com/docs/get/users/:user_id/conversations>
+
+=head2 get_conversation
+
+  $res = $xing->get_conversation(
+    user_id => $user_id, id => $conversation_id,
+    user_fields => \@user_fields, with_latest_messages => $number
+  );
+
+See L<https://dev.xing.com/docs/get/users/:user_id/conversations/:id>
+
+=head2 list_conversation_messages
+
+  $res = $xing->list_conversation_messages(
+    user_id => $user_id, conversation_id => $conversation_id,
+    limit => $limit, offset => $offset, user_fields => \@user_fields
+  );
+
+See L<https://dev.xing.com/docs/get/users/:user_id/conversations/:conversation_id/messages>
+
+=head2 get_conversation_message
+
+  $res = $xing->get_conversation_message(
+    user_id => $user_id, conversation_id => $conversation_id,
+    id => $message_id, user_fields => \@user_fields
+  );
+
+See L<https://dev.xing.com/docs/get/users/:user_id/conversations/:conversation_id/messages/:id>
+
 =head2 create_status_message
 
   $res = $xing->create_status_message(id => $id, message => $message);
@@ -862,52 +934,52 @@ See L<https://dev.xing.com/docs/post/users/:id/status_message>
 
 =head2 get_profile_message
 
-  $res = $xing->get_profile_message(user_id => $id);
+  $res = $xing->get_profile_message(user_id => $user_id);
 
 See L<https://dev.xing.com/docs/get/users/:user_id/profile_message>
 
 =head2 update_profile_message
 
   $res = $xing->update_profile_message(
-    user_id => $id, message => $message, public => $bool
+    user_id => $user_id, message => $message, public => $bool
   );
 
 See L<https://dev.xing.com/docs/put/users/:user_id/profile_message>
 
-=head2 get_contacts
+=head2 list_contacts
 
-  $res = $xing->get_contacts(
-    user_id => $id,
+  $res = $xing->list_contacts(
+    user_id => $user_id,
     limit => $limit, offset => $offset, order_by => $order_by,
     user_fields => \@user_fields
   );
 
 See L<https://dev.xing.com/docs/get/users/:user_id/contacts>
 
-=head2 get_shared_contacts
+=head2 list_shared_contacts
 
-  $res = $xing->get_shared_contacts(
-    user_id => $id,
+  $res = $xing->list_shared_contacts(
+    user_id => $user_id,
     limit => $limit, offset => $offset, order_by => $order_by,
     user_fields => \@user_fields
   );
 
 See L<https://dev.xing.com/docs/get/users/:user_id/contacts/shared>
 
-=head2 get_incoming_contact_requests
+=head2 list_incoming_contact_requests
 
-  $res = $xing->get_incoming_contact_requests(
-    user_id => $id,
+  $res = $xing->list_incoming_contact_requests(
+    user_id => $user_id,
     limit => $limit, offset => $offset,
     user_fields => \@user_fields
   );
 
 See L<https://dev.xing.com/docs/get/users/:user_id/contact_requests>
 
-=head2 get_sent_contact_requests
+=head2 list_sent_contact_requests
 
-  $res = $xing->get_sent_contact_requests(
-    user_id => $id, limit => $limit, offset => $offset
+  $res = $xing->list_sent_contact_requests(
+    user_id => $user_id, limit => $limit, offset => $offset
   );
 
 See L<https://dev.xing.com/docs/get/users/:user_id/contact_requests/sent>
@@ -915,7 +987,7 @@ See L<https://dev.xing.com/docs/get/users/:user_id/contact_requests/sent>
 =head2 create_contact_request
 
   $res = $xing->create_contact_request(
-    user_id => $id, message => $message
+    user_id => $user_id, message => $message
   );
 
 See L<https://dev.xing.com/docs/post/users/:user_id/contact_requests>
@@ -939,7 +1011,7 @@ See L<https://dev.xing.com/docs/delete/users/:user_id/contact_requests/:id>
 =head2 get_contact_paths
 
   $res = $xing->get_contact_paths(
-    user_id => $id,
+    user_id => $user_id,
     other_user_id => $other_user_id,
     all_paths => $bool,
     user_fields => \@user_fields
@@ -947,10 +1019,10 @@ See L<https://dev.xing.com/docs/delete/users/:user_id/contact_requests/:id>
 
 See L<https://dev.xing.com/docs/get/users/:user_id/network/:other_user_id/paths>
 
-=head2 get_bookmarks
+=head2 list_bookmarks
 
-  $res = $xing->get_bookmarks(
-    user_id => $id,
+  $res = $xing->list_bookmarks(
+    user_id => $user_id,
     limit => $limit, offset => $offset,
     user_fields => \@user_fields
   );
@@ -959,27 +1031,27 @@ See L<https://dev.xing.com/docs/get/users/:user_id/bookmarks>
 
 =head2 create_bookmark
 
-  $res = $xing->create_bookmark(id => $id, user_id => $id);
+  $res = $xing->create_bookmark(id => $id, user_id => $user_id);
 
 See L<https://dev.xing.com/docs/put/users/:user_id/bookmarks/:id>
 
 =head2 delete_bookmark
 
-  $res = $xing->delete_bookmark(id => $id, user_id => $id);
+  $res = $xing->delete_bookmark(id => $id, user_id => $user_id);
 
 See L<https://dev.xing.com/docs/delete/users/:user_id/bookmarks/:id>
 
 =head2 get_network_feed
 
   $res = $xing->get_network_feed(
-    user_id => $id,
+    user_id => $user_id,
     aggregate => $bool,
     since => $date,
     user_fields => \@user_fields
   );
 
   $res = $xing->get_network_feed(
-    user_id => $id,
+    user_id => $user_id,
     aggregate => $bool,
     until => $date,
     user_fields => \@user_fields
@@ -990,13 +1062,13 @@ See L<https://dev.xing.com/docs/get/users/:user_id/network_feed>
 =head2 get_user_feed
 
   $res = $xing->get_user_feed(
-    user_id => $id,
+    user_id => $user_id,
     since => $date,
     user_fields => \@user_fields
   );
 
   $res = $xing->get_user_feed(
-    user_id => $id,
+    user_id => $user_id,
     until => $date,
     user_fields => \@user_fields
   );
@@ -1021,9 +1093,9 @@ See L<https://dev.xing.com/docs/post/activities/:id/share>
 
 See L<https://dev.xing.com/docs/delete/activities/:id>
 
-=head2 get_activity_comments
+=head2 list_activity_comments
 
-  $res = $xing->get_activity_comments(
+  $res = $xing->list_activity_comments(
     activity_id => $activity_id,
     limit => $limit, offset => $offset,
     user_fields => \@user_fields
@@ -1049,9 +1121,9 @@ See L<https://dev.xing.com/docs/post/activities/:activity_id/comments>
 
 See L<https://dev.xing.com/docs/delete/activities/:activity_id/comments/:id>
 
-=head2 get_activity_likes
+=head2 list_activity_likes
 
-  $res = $xing->get_activity_likes(
+  $res = $xing->list_activity_likes(
     activity_id => $activity_id,
     limit => $limit, offset => $offset,
     user_fields => \@user_fields
@@ -1071,27 +1143,27 @@ See L<https://dev.xing.com/docs/put/activities/:activity_id/like>
 
 See L<https://dev.xing.com/docs/delete/activities/:activity_id/like>
 
-=head2 get_profile_visits
+=head2 list_profile_visits
 
-  $res = $xing->create_profile_visit(
-    user_id => $id,
-    limit => $limit, offset => $offset,
-    since => $date,
-    strip_html => $bool
-  );
+  $res = $xing->list_profile_visits(user_id => $user_id);
 
 See L<https://dev.xing.com/docs/get/users/:user_id/visits>
 
 =head2 create_profile_visit
 
-  $res = $xing->get_profile_visits(user_id => $id);
+  $res = $xing->create_profile_visit(
+    user_id => $user_id,
+    limit => $limit, offset => $offset,
+    since => $date,
+    strip_html => $bool
+  );
 
 See L<https://dev.xing.com/docs/post/users/:user_id/visits>
 
-=head2 get_recommendations
+=head2 list_recommendations
 
-  $res = $xing->get_recommendations(
-    user_id => $id,
+  $res = $xing->list_recommendations(
+    user_id => $user_id,
     limit => $limit, offset => $offset,
     similar_user_id => $similar_user_id,
     user_fields => \@user_fields
@@ -1099,6 +1171,14 @@ See L<https://dev.xing.com/docs/post/users/:user_id/visits>
 
 See L<https://dev.xing.com/docs/get/users/:user_id/network/recommendations>
 
+=head2 delete_recommendation
+
+  $res = $xing->list_recommendation(
+    user_id => $user_id, id => $delete_user_id,
+  );
+
+See L<https://dev.xing.com/docs/delete/users/:user_id/network/recommendations/user/:id>
+
 =head2 create_invitations
 
   $res = $xing->create_invitations(
@@ -1112,7 +1192,7 @@ See L<https://dev.xing.com/docs/post/users/invite>
 =head2 update_geo_location
 
   $res = $xing->update_geo_location(
-    user_id => $id,
+    user_id => $user_id,
     accuracy => $accuracy,
     latitude => $latitude, longitude => $longitude,
     ttl => $ttl
@@ -1120,10 +1200,10 @@ See L<https://dev.xing.com/docs/post/users/invite>
 
 See L<https://dev.xing.com/docs/put/users/:user_id/geo_location>
 
-=head2 get_nearby_users
+=head2 list_nearby_users
 
-  $res = $xing->get_nearby_users(
-    user_id => $id,
+  $res = $xing->list_nearby_users(
+    user_id => $user_id,
     age => $age,
     radius => $radius,
     user_fields => \@user_fields
@@ -1153,6 +1233,57 @@ A list of named arguments, e.g. C<< id => 'me', text => 'Blah!' >>.
 
 =back
 
+=head1 DEPRECATED METHODS
+
+For the sake of consistency a couple of API methods have been renamed.
+These methods are still available under their old names. The old names
+are not detectable by means of L</Method Introspection>, and will be
+removed in a future release.
+
+=head2 get_contacts
+
+Renamed to L</list_contacts>.
+
+=head2 get_shared_contacts
+
+Renamed to L</list_shared_contacts>.
+
+=head2 get_incoming_contact_requests
+
+Renamed to L</list_incoming_contact_requests>.
+
+=head2 get_sent_contact_requests
+
+Renamed to L</list_sent_contact_requests>.
+
+=head2 get_bookmarks
+
+Renamed to L</list_bookmarks>.
+
+=head2 get_activity_comments
+
+Renamed to L</list_activity_comments>.
+
+=head2 get_activity_likes
+
+Renamed to L</list_activity_likes>.
+
+=head2 get_profile_visits
+
+Renamed to L</list_profile_visits>.
+
+=head2 get_recommendations
+
+Renamed to L</list_recommendations>.
+
+=head2 block_recommendation
+
+Renamed to L</delete_recommendation>.
+
+=head2 get_nearby_users
+
+Renamed to L</list_nearby_users>.
+
 =head1 SEE ALSO
 
 L<WebService::XING::Response>, L<WebService::XING::Function>,
